"use strict";
var Sequelize = require('sequelize');
var pretur_spec_1 = require('pretur.spec');
function buildSequelizeModel(spec, sequelize, options) {
    var attributes = {};
    spec.attributeArray.forEach(function (attrib) {
        return attributes[attrib.name] = {
            allowNull: !attrib.required,
            autoIncrement: attrib.primary && !!attrib.autoIncrement,
            defaultValue: attrib.defaultValue,
            field: (options &&
                options.attributeToFieldMap &&
                options.attributeToFieldMap[attrib.name]) || undefined,
            primaryKey: !!attrib.primary,
            type: datatypeToSequelizeType(attrib.type),
            unique: !attrib.primary && !!attrib.unique,
            values: pretur_spec_1.EnumType.is(attrib.type) ? attrib.type.values.map(function (v) { return v.name; }) : undefined,
        };
    });
    var defineOptions = {
        tableName: (options && options.tableName) || spec.name,
    };
    if (spec.indexes.unique) {
        defineOptions.indexes = spec.indexes.unique.map(function (fields) { return ({ unique: true, fields: fields }); });
    }
    var model = sequelize.define(spec.name, attributes, defineOptions);
    function initialize(pool) {
        spec.nonVirtualRelations.superclass.forEach(function (superclass) {
            if (pool.models[superclass.model] && pool.models[superclass.model].sequelizeModel) {
                model.belongsTo(pool.models[superclass.model].sequelizeModel, {
                    as: superclass.alias,
                    foreignKey: superclass.key,
                    onDelete: superclass.onDelete,
                    onUpdate: superclass.onUpdate,
                });
            }
        });
        spec.nonVirtualRelations.subclass.forEach(function (subclass) {
            if (pool.models[subclass.model] && pool.models[subclass.model].sequelizeModel) {
                model.hasOne(pool.models[subclass.model].sequelizeModel, {
                    as: subclass.alias,
                    foreignKey: subclass.key,
                    onDelete: subclass.onDelete,
                    onUpdate: subclass.onUpdate,
                });
            }
        });
        spec.nonVirtualRelations.manyToMany.forEach(function (manyToMany) {
            if (pool.models[manyToMany.model] && pool.models[manyToMany.model].sequelizeModel) {
                model.belongsToMany(pool.models[manyToMany.model].sequelizeModel, {
                    as: manyToMany.alias,
                    foreignKey: manyToMany.key,
                    onDelete: manyToMany.onDelete,
                    onUpdate: manyToMany.onUpdate,
                    through: pool.models[manyToMany.through].sequelizeModel,
                });
            }
        });
        spec.nonVirtualRelations.master.forEach(function (master) {
            if (pool.models[master.model] && pool.models[master.model].sequelizeModel) {
                model.belongsTo(pool.models[master.model].sequelizeModel, {
                    as: master.alias,
                    foreignKey: master.key,
                    onDelete: master.onDelete,
                    onUpdate: master.onUpdate,
                });
            }
        });
        spec.nonVirtualRelations.detail.forEach(function (detail) {
            if (pool.models[detail.model] && pool.models[detail.model].sequelizeModel) {
                model.hasMany(pool.models[detail.model].sequelizeModel, {
                    as: detail.alias,
                    foreignKey: detail.key,
                    onDelete: detail.onDelete,
                    onUpdate: detail.onUpdate,
                });
            }
        });
        spec.nonVirtualRelations.injective.forEach(function (injective) {
            if (pool.models[injective.model] && pool.models[injective.model].sequelizeModel) {
                model.hasOne(pool.models[injective.model].sequelizeModel, {
                    as: injective.alias,
                    foreignKey: injective.key,
                    onDelete: injective.onDelete,
                    onUpdate: injective.onUpdate,
                });
            }
        });
        spec.nonVirtualRelations.recursive.forEach(function (recursive) {
            if (pool.models[recursive.model] && pool.models[recursive.model].sequelizeModel) {
                model.hasOne(pool.models[recursive.model].sequelizeModel, {
                    as: recursive.alias,
                    foreignKey: recursive.key,
                    onDelete: recursive.onDelete,
                    onUpdate: recursive.onUpdate,
                });
            }
        });
        if (options && options.createDatabase) {
            options.createDatabase(model);
        }
    }
    return { sequelizeModel: model, initialize: initialize };
}
exports.buildSequelizeModel = buildSequelizeModel;
function datatypeToSequelizeType(datatype) {
    switch (true) {
        case pretur_spec_1.IntegerType.is(datatype):
            return Sequelize.INTEGER;
        case pretur_spec_1.StringType.is(datatype):
            return Sequelize.TEXT;
        case pretur_spec_1.ObjectType.is(datatype):
            return Sequelize.JSONB;
        case pretur_spec_1.EnumType.is(datatype):
            return Sequelize.ENUM;
        case pretur_spec_1.BooleanType.is(datatype):
            return Sequelize.BOOLEAN;
        case pretur_spec_1.DoubleType.is(datatype):
            return Sequelize.DOUBLE;
        case pretur_spec_1.DateType.is(datatype):
            return Sequelize.DATE;
        case pretur_spec_1.RangeType.is(datatype):
            return Sequelize.RANGE(datatypeToSequelizeType(datatype.subType));
    }
}
